@@ -28,8 +28,9 @@
 #include "braille.h"
 #include "baumbrl.h"
 #include "alvabrl.h"
+#ifdef BRLTTY_SUPPORT
 #include "ttybrl.h"
-
+#endif
 
 typedef struct
 {
@@ -46,6 +47,7 @@ BRLDEV_INFO   SupportedDevices[] = {
   {"VARIO40", "BAUM VARIO - 40 cells"},
   {"VARIO80", "BAUM VARIO - 80 cells"},
   {"DM80P", "BAUM DM80 - 80 cells"},
+  {"INKA","BAUM INKA"},
   {"ALVA380", "ALVABRAILLE 380"},
   {"ALVA544", "ALVABRAILLE 544"},
   {"ALVA570", "ALVABRAILLE 570"},
