@@ -61,6 +61,7 @@ gint srcore_notify_id;
 **/
 gboolean 	exitackget;
 
+gboolean 	init_finish = FALSE;
 /**
  *
  * Create a new gconf client for SRCORE part
@@ -489,12 +490,19 @@ srcore_changes_cb (GConfClient	*client,
     
     key = (gchar*) gconf_entry_get_key (entry);
     
-    if (strcmp (key, CONFIG_PATH SRCORE_KEY_PATH SRCORE_EXIT_ACK_KEY) == 0)
+    if (!strcmp (key, CONFIG_PATH SRCORE_KEY_PATH SRCORE_EXIT_ACK_KEY))
     {
 	exitackget = gconf_value_get_bool (entry->value);
 	return;
     }    
     
+    if (!strcmp (key, CONFIG_PATH SRCORE_KEY_PATH SRCORE_INITIALIZATION_END))
+    {
+	init_finish = TRUE;
+	return;
+    }    
+    
+    
     sru_debug ("coreconf:srcore_changes_cb:Entry key:%s",gconf_entry_get_key(entry));
     sru_debug ("coreconf:srcore_changes_cb:Entry value type:%i",entry->value->type);
     switch(entry->value->type)
