@@ -40,8 +40,7 @@ gnopiconf_gconf_client_init (void)
  
     if (gnopernicus_client == NULL) 
     {
-	sru_warning (_("gnopernicus:Failed to init GConf client:\n"));
-	sru_warning (_("Recommended to delete ~/.gconf and ~/.gconfd directories"));
+	sru_warning (_("Failed to init gconf client."));
 	return FALSE;
     }
     
@@ -60,8 +59,7 @@ gnopiconf_client_add_dir (const gchar *path)
 
     if (error != NULL)
     {
-	sru_warning (_("gnopiconf:Failed to add directory:%s\n"),path);
-	sru_warning (_("Recommended to delete ~/.gconf and ~/.gconfd directories"));
+	sru_warning (_(error->message));
 	g_error_free (error);
 	error = NULL;
 	return FALSE;
@@ -125,8 +123,14 @@ gnopiconf_value_change (const gchar *key,
     gconf_value_set_string (value, val);
     gconf_client_set (gnopernicus_client, key, value, &error);
     gconf_value_free (value);
-    if (error)
-	fprintf (stderr, "Error in set");
+    if (error != NULL)
+    {
+	sru_warning (_(error->message));
+	g_error_free (error);
+	error = NULL;
+    }
+    
+
 }
 
 static gboolean
@@ -167,8 +171,7 @@ gnopiconf_set_int (gint val,
 
     if (error != NULL)
     {
-	sru_warning (_("gnopernicus:Failed to set value:%d\n"), val);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_(error->message));
 	g_error_free (error);
 	error = NULL;
     }
@@ -184,7 +187,7 @@ gnopiconf_set_int_in_section (gint val,
     gchar *path = NULL;
     sru_return_val_if_fail (key != NULL, FALSE);
     sru_return_val_if_fail (section != NULL, FALSE);
-    path = gconf_concat_dir_and_key ( section, key);
+    path = gconf_concat_dir_and_key (section, key);
     gnopiconf_set_int (val, path);
     g_free (path);
     return TRUE;
@@ -213,12 +216,10 @@ gnopiconf_set_string (const gchar *val,
 
     if (error != NULL)
     {
-	sru_warning (_("gnopernicus:Failed to set value:%s\n"), val);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_(error->message));
 	g_error_free (error);
 	error = NULL;
     }
-gconf_client_suggest_sync (gnopernicus_client, NULL);        
     return ret;
 }
 
@@ -255,8 +256,7 @@ gnopiconf_set_bool (gboolean val,
 
     if (error != NULL)
     {
-	sru_warning (_("gnopernicus:Failed to set value:%d\n"), val);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_(error->message));
 	g_error_free (error);
 	error = NULL;
     }
@@ -310,10 +310,9 @@ gnopiconf_get_int_with_default (const gchar *key,
     	    ret_val = gconf_value_get_int (value);
         else
 	{
-    	    sru_warning (_("gnopernicus:Invalid type of key %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+    	    sru_warning (_("Invalid type of key: %s."), key);
 	    if (!gnopiconf_set_int (ret_val, key))
-	        sru_warning (_("gnopernicus:Failed to set value %d\n"), ret_val);
+	        sru_warning (_("Failed to set value: %d."), ret_val);
 	}
 	    
 	gconf_value_free (value);
@@ -324,12 +323,14 @@ gnopiconf_get_int_with_default (const gchar *key,
     {
 	if (error != NULL)
 	{
-	    sru_warning (_("gnopernicus:Failed to get value %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	    sru_warning (_(error->message));
+	    g_error_free (error);
+	    error = NULL;
 	}
+
 	    
 	if (!gnopiconf_set_int (default_value, key))
-	    sru_warning (_("gnopernicus:Failed to set value %d\n"), default_value);
+	    sru_warning (_("Failed to set value: %d."), default_value);
 		
         return default_value;
     }    
@@ -374,14 +375,13 @@ gnopiconf_get_string_with_default (const gchar *key,
     {    
 	if (error != NULL)
 	{
-	    sru_warning (_("gnopernicus:Failed return string value %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	    sru_warning (_(error->message));
 	    g_error_free (error);
 	    error = NULL;
 	}
 	
 	if (!gnopiconf_set_string (default_value, key))
-	    sru_warning (_("gnopernicus:Failed to set value %s\n"), default_value);
+	    sru_warning (_("Failed to set value: %s."), default_value);
 	
 	return g_strdup (default_value);
     }
@@ -389,13 +389,12 @@ gnopiconf_get_string_with_default (const gchar *key,
     {
 	if (error != NULL)
 	{
-	    sru_warning(_("gnopernicus:Failed to get value %s\n"), key);
-	    sru_warning(_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	    sru_warning(_(error->message));
 	    g_error_free (error);
 	    error = NULL;
 		
 	    if (!gnopiconf_set_string (default_value, key))
-		sru_warning (_("gnopernicus:Failed to set value %s\n"), retval);
+		sru_warning (_("Failed to set value: %s."), retval);
 	    return g_strdup (default_value);
 	}
 	
@@ -446,10 +445,9 @@ gnopiconf_get_bool_with_default (const gchar* key,
     	    retval = gconf_value_get_bool (val);
         else
 	{
-    	    sru_warning (_("gnopernicus:Invalid type of key %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+    	    sru_warning (_("Invalid type of key: %s."), key);
 	    if (!gnopiconf_set_bool (retval, key))
-	        sru_warning (_("gnopernicus:Failed to set value %d\n"), retval);
+	        sru_warning (_("Failed to set value: %d."), retval);
 	}
 	
         gconf_value_free (val);
@@ -459,12 +457,13 @@ gnopiconf_get_bool_with_default (const gchar* key,
     {
 	if (error != NULL)
 	{
-	    sru_warning (_("gnopernicus:Failed to get value %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	    sru_warning (_(error->message));
+	    g_error_free (error);
+	    error = NULL;
 	}
 	    
 	if (!gnopiconf_set_bool (default_value, key))
-		sru_warning (_("gnopernicus:Failed to set value %d\n"), default_value);
+		sru_warning (_("Failed to set value: %d."), default_value);
         return default_value;
     }
 }
@@ -476,7 +475,7 @@ gnopiconf_get_bool_from_section_with_default (const gchar *section,
 {
     gchar *path = NULL;
     gboolean return_value;
-    sru_return_val_if_fail (key != NULL, 	default_value);
+    sru_return_val_if_fail (key != NULL, default_value);
     sru_return_val_if_fail (section != NULL, default_value);
     path = gconf_concat_dir_and_key (section, key);
     return_value = gnopiconf_get_bool_with_default (path, default_value);
@@ -485,6 +484,43 @@ gnopiconf_get_bool_from_section_with_default (const gchar *section,
 }
 
 
+gboolean
+gnopiconf_get_bool (const gchar* key,
+		    GError **error)
+{
+    GConfValue* val = NULL;
+    gchar *key_error = NULL;
+
+    sru_return_val_if_fail (key != NULL, FALSE);
+    sru_return_val_if_fail (gnopernicus_client != NULL, FALSE);
+    
+    gconf_client_add_dir (gnopernicus_client,
+			 "/desktop/gnome/interface", 
+			 GCONF_CLIENT_PRELOAD_NONE, 
+			 error);
+
+    if (*error != NULL)
+	return FALSE;
+    
+    if (!gconf_valid_key (key, &key_error))
+    {
+	sru_error (key_error);
+	g_free (key_error);
+	return FALSE;
+    }    
+
+    val = gconf_client_get (gnopernicus_client, key, error);
+
+    
+    if (*error)
+	return FALSE;
+	
+      
+    if (gnopiconf_check_type (key, val, GCONF_VALUE_BOOL, error))
+    	return gconf_value_get_bool (val);
+    
+    return FALSE;
+}
 
 
 
@@ -509,8 +545,7 @@ gnopiconf_set_list (GSList *list,
 
     if (error != NULL)
     {
-	sru_warning (_("gnopernicus:Failed to set list with type %d\n"), list_type);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_(error->message));
 	g_error_free (error);
 	error = NULL;
     }
@@ -583,10 +618,9 @@ gnopiconf_get_list_with_default (const gchar* key,
         }	
         else
     	{
-    	    sru_warning (_("cmdmapconf:Invalid type of key %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+    	    sru_warning (_("Invalid type of key: %s."), key);
 	    if (!gnopiconf_set_list (retval, *type, key))
-		    sru_warning (_("cmdmapconf:Failed to set value\n"));
+		    sru_warning (_("Failed to set value."));
 	}
 	
         gconf_value_free (val);
@@ -596,10 +630,11 @@ gnopiconf_get_list_with_default (const gchar* key,
     {
 	if (error != NULL)
 	{
-	    sru_warning (_("cmdmapconf:Failed to get value %s\n"), key);
-	    sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	    sru_warning (_(error->message));
+	    g_error_free (error);
+	    error = NULL;
 	}
-	    
+		    
         return NULL;
     }
 }
@@ -613,6 +648,7 @@ gnopiconf_unset_key (const gchar *key)
     gchar *key_error = NULL;
     
     sru_return_val_if_fail ( key != NULL, FALSE);
+    
     if (!gconf_valid_key (key, &key_error))
     {
 	sru_error (key_error);
@@ -631,7 +667,7 @@ gnopiconf_unset_key (const gchar *key)
         
 	if (error != NULL)
 	{
-	    sru_warning("gnopernicus:Failed unset key");
+	    sru_warning(_(error->message));
 	    g_error_free(error);
 	    error = NULL;
 	}
@@ -655,7 +691,7 @@ gnopiconf_remove_dir (gchar *dir)
     
     if (err)
     {
-	sru_warning("gnopernicus:Failed to remove dir");
+	sru_warning(_(err->message));
 	g_error_free (err);
 	return FALSE;
     }
@@ -678,8 +714,7 @@ gnopiconf_get_all_entries (const gchar *path)
     
     if (error)
     {
-        sru_warning (_("presentation:Failed to load items from directory:\n"));
-        sru_warning (_("Recommended to delete ~/.gconf and ~/.gconfd directories"));
+        sru_warning (_(error->message));
         g_error_free (error);
         error = NULL;
 	return NULL;
