@@ -27,10 +27,14 @@
 #include "gnopiui.h"
 
 
-const gchar *magui_smoothing[] =
+struct
 {
-    "none",
-    "bilinear"
+    gchar *description;
+    gchar *key;
+} magui_smoothing[] =
+{
+    {N_("none"),	"none"},
+    {N_("bilinear"), 	"bilinear"}
 };
 
 struct
@@ -39,10 +43,10 @@ struct
     gchar *key;
 } magui_mouse_tracking [] = 
 {
-    {"push", 		"mouse-push"},
-    {"centered",	"mouse-centered"},
-    {"proportional",	"mouse-proportional"},
-    {"none",		"none"}
+    {N_("push"), 		"mouse-push"},
+    {N_("centered"),		"mouse-centered"},
+    {N_("proportional"),	"mouse-proportional"},
+    {N_("none"),		"none"}
 };
 
 typedef enum
@@ -104,9 +108,6 @@ GtkWidget    	*sp_zp_left,
 GtkWidget	*cp_border_color,
 		*cp_crosswire_color,
 		*cp_cursor_color;
-GtkWidget	*im_cursor_color,
-		*im_crosswire_color;
-		
 
 void 
 magui_cursor_off (gboolean val)
@@ -115,7 +116,6 @@ magui_cursor_off (gboolean val)
     gtk_widget_set_sensitive ( GTK_WIDGET (cp_cursor_color), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (sp_cursor_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_cursor_color), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (im_cursor_color), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (sp_cursor_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_cursor_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (ck_crosswire), val);    
@@ -129,7 +129,6 @@ magui_cursor_off (gboolean val)
     gtk_widget_set_sensitive ( GTK_WIDGET (cp_crosswire_color), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_color), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (im_crosswire_color), val);
 }
 
 void 
@@ -140,7 +139,6 @@ magui_crosswire_off (gboolean val)
     gtk_widget_set_sensitive ( GTK_WIDGET (cp_crosswire_color), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_size), val);
     gtk_widget_set_sensitive ( GTK_WIDGET (lb_crosswire_color), val);
-    gtk_widget_set_sensitive ( GTK_WIDGET (im_crosswire_color), val);
 }
 
 
@@ -432,7 +430,7 @@ void
 magui_zoomer_remove_clicked(GtkWidget *widget,
 			    gpointer data)
 {
-    sru_message (_("\nCan't remove zoomer because the multiple zoomers are not supported yet."));
+    sru_message (_("Can NOT remove zoomer because multiple zoomers are not supported in this version."));
 }
 
 static GtkTreeModel*
@@ -578,8 +576,6 @@ magui_set_handlers_magnifier_settings (GladeXML *xml)
     lb_crosswire_size	= glade_xml_get_widget (xml, "lb_crosswire_size");
     lb_crosswire_color	= glade_xml_get_widget (xml, "lb_crosswire_color");
 
-    im_cursor_color	= glade_xml_get_widget (xml, "im_cursor_color");
-    im_crosswire_color	= glade_xml_get_widget (xml, "im_crosswire_color");
 
     gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_cursor_size), MIN_CURSOR_SIZE, MAX_CURSOR_SIZE);
     gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_crosswire_size), MIN_CROSSWIRE_SIZE, MAX_CROSSWIRE_SIZE);
@@ -899,10 +895,11 @@ magui_magnification_options_save_changed (void)
     {
 	for (iter = 0; iter < G_N_ELEMENTS (magui_smoothing); iter++)
 	{
-	    if (strcmp (magui_smoothing[iter], tmp) == 0)
+	    if (strcmp (magui_smoothing[iter].description, tmp) == 0)
 	    {
 		g_free (magnifier_setting->smoothing);
-		magnifier_setting->smoothing = g_strdup (tmp);
+		magnifier_setting->smoothing = 
+		    g_strdup (magui_smoothing[iter].key);
 		break;
 	    }
 	}
@@ -1207,10 +1204,11 @@ magui_smoothing_value_changed(GtkWidget *widget,
     {
 	for (iter = 0; iter < G_N_ELEMENTS (magui_smoothing); iter++)
 	{
-	    if (strcmp (magui_smoothing[iter], tmp) == 0)
+	    if (strcmp (magui_smoothing[iter].description, tmp) == 0)
 	    {
 		g_free (magnifier_setting->smoothing);
-		magnifier_setting->smoothing = g_strdup (tmp);
+		magnifier_setting->smoothing = 
+		    g_strdup (magui_smoothing[iter].key);
 		magconf_save_item_in_schema (DEFAULT_MAGNIFIER_SCHEMA,
 					    GCONF_VALUE_STRING,
 					    MAGNIFIER_SMOOTHING,
@@ -1408,10 +1406,14 @@ magui_set_handlers_magnification_options(GladeXML *xml)
     gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_factory), MIN_ZOOM_FACTOR, MAX_ZOOM_FACTOR);
 
     magconf_get_display_size (&display_size.size_x, &display_size.size_y);
-    gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_zp_left),  DEFAULT_SCREEN_MIN_SIZE, display_size.size_x);
-    gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_zp_top),   DEFAULT_SCREEN_MIN_SIZE, display_size.size_y);
-    gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_zp_width), DEFAULT_SCREEN_MIN_SIZE, display_size.size_x);
-    gtk_spin_button_set_range ( GTK_SPIN_BUTTON (sp_zp_height),DEFAULT_SCREEN_MIN_SIZE, display_size.size_y);
+    
+    gtk_spin_button_set_range (GTK_SPIN_BUTTON (sp_zp_left),  DEFAULT_SCREEN_MIN_SIZE, display_size.size_x);
+    gtk_spin_button_set_range (GTK_SPIN_BUTTON (sp_zp_top),   DEFAULT_SCREEN_MIN_SIZE, display_size.size_y);
+    gtk_spin_button_set_range (GTK_SPIN_BUTTON (sp_zp_width), DEFAULT_SCREEN_MIN_SIZE, display_size.size_x);
+    gtk_spin_button_set_range (GTK_SPIN_BUTTON (sp_zp_height),DEFAULT_SCREEN_MIN_SIZE, display_size.size_y);
+    
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_smoothing)->entry), FALSE);
+    gtk_entry_set_editable (GTK_ENTRY (GTK_COMBO (cb_mouse_tracking)->entry), FALSE);
     
     glade_xml_signal_connect (xml, "on_et_source_activate",		
 			    GTK_SIGNAL_FUNC (magui_source_changed));
@@ -1486,7 +1488,7 @@ magui_load_magnification_options_interface ()
     
     magui_magnification_options_value_add_to_widgets (magnifier_setting);
 
-    zoomer_title = g_strdup_printf ("Zoomer options (%s)", selected_zoomer);
+    zoomer_title = g_strdup_printf (_("Zoomer Options (%s)"), selected_zoomer);
     gtk_window_set_title (GTK_WINDOW (w_magnification_options), zoomer_title);
     g_free (zoomer_title);
     
@@ -1661,10 +1663,24 @@ magui_set_smoothing (Magnifier *magnifier_setting)
     sru_return_val_if_fail (magnifier_setting, FALSE);
     if (!w_magnification_options) 
 	return FALSE;
-    
-    if (magnifier_setting->smoothing)
-	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cb_smoothing)->entry), 
-			    magnifier_setting->smoothing);
+        
+    if (magnifier_setting->smoothing &&
+	strlen (magnifier_setting->smoothing) != 0)
+    {
+	gint iter;
+	for (iter= 0 ; iter < G_N_ELEMENTS (magui_smoothing) ; ++iter)
+	{
+	    if (!strcmp (magui_smoothing[iter].key, 
+			 magnifier_setting->smoothing))
+	    {
+		gtk_entry_set_text  (GTK_ENTRY 
+				    (GTK_COMBO 
+				    (cb_smoothing)->entry), 
+				    magui_smoothing[iter].description);
+		break;
+	    }
+	}
+    }
     else
 	gtk_entry_set_text (GTK_ENTRY (GTK_COMBO (cb_smoothing)->entry), 
 			    DEFAULT_MAGNIFIER_SMOOTHING);
