@@ -37,25 +37,32 @@ void	spconf_changes_cb	(GConfClient	*client,
 				gpointer	user_data);
 
 extern Speech 	*speech_setting;
-static guint sp_notify_id;
+static guint 	sp_notify_id;
 
+/**
+ *
+ * Default values for voices.
+ *
+**/
 SpeakerSettings speakers[]=
 {
-    {"accelerator",	DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"childcount",	DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"location",	DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"message",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"name",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"role",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"shortcut",	DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"state",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"system",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"text",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
-    {"value",		DEFAULT_SPEECH_ENGINE_DRIVER, DEFAULT_SPEECH_ENGINE_VOICE,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH}
+    {"accelerator",	DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"childcount",	DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"empty",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"location",	DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"message",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"name",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"role",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"shortcut",	DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"state",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"system",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"text",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH},
+    {"value",		DEFAULT_SPEECH_ENGINE_DRIVER, NULL, DEFAULT_SPEECH_ENGINE_VOICE, NULL,	DEFAULT_SPEECH_VOLUME,	DEFAULT_SPEECH_RATE,	DEFAULT_SPEECH_PITCH}
 };
 
 extern GConfClient *gnopernicus_client;
-DictionaryListType		*dictionary_list = NULL;
+DictionaryListType *dictionary_list = NULL;
+
 /**
  *
  * Create a new gconf client for Speech settings
@@ -75,8 +82,8 @@ spconf_gconf_client_init (void)
 			    NULL,NULL,&error);
     if (error != NULL)
     {
-        sru_warning (_("spconf:Failed to add notify:\n"));
-        sru_warning (_("Recommended to delete ~/.gconf and ~/.gconfd directories"));
+        sru_warning (_("Failed to add notify."));
+	sru_warning (_(error->message));
         g_error_free (error);
         error = NULL;
         return FALSE;
@@ -138,7 +145,7 @@ spconf_setting_new (void)
     new_speech = (Speech *) g_new0 (Speech, 1) ; 
     
     if (!new_speech) 
-	sru_error (_("Unable to allocate memory"));
+	sru_error (_("Unable to allocate memory."));
     
     return new_speech ; 
 } 
@@ -321,7 +328,7 @@ spconf_dictionary_changes (void)
 {
     gnopiconf_unset_key (CONFIG_PATH SPEECH_VOICE_KEY_PATH SPEECH_DICTIONARY_CHANGES);
     if (!gnopiconf_set_bool (TRUE, CONFIG_PATH SPEECH_VOICE_KEY_PATH SPEECH_DICTIONARY_CHANGES))
-	sru_warning (_("spconf:Failed to set dictionary changes\n"));
+	sru_warning (_("Failed to set dictionary changes."));
 }
 
 void
@@ -329,7 +336,7 @@ spconf_speech_voice_removed (const gchar *voice)
 {
     gnopiconf_unset_key (CONFIG_PATH SPEECH_VOICE_KEY_PATH SPEECH_VOICE_REMOVED);
     if (!gnopiconf_set_string (voice, CONFIG_PATH SPEECH_VOICE_KEY_PATH SPEECH_VOICE_REMOVED))
-	sru_warning (_("spconf:Failed to set value\n"));
+	sru_warning (_("Failed to set removed voice value."));
 }
 
 
@@ -339,8 +346,7 @@ spconf_count_set (const gchar *count)
 {
     if (!gnopiconf_set_string (count, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_COUNT))
     {
-	sru_warning (_("spconf:Failed to set Count:%s\n"),count);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set count: %s."),count);
     }
 }
 
@@ -349,8 +355,7 @@ spconf_punctuation_set (const gchar *punctuation)
 {
     if (!gnopiconf_set_string (punctuation, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_PUNCTUATION))
     {
-	sru_warning (_("spconf:Failed to set Punctuation:%s\n"),punctuation);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set Punctuation: %s."),punctuation);
     }
 }
 
@@ -359,8 +364,7 @@ spconf_text_echo_set (const gchar *text_echo)
 {
     if (!gnopiconf_set_string (text_echo, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_TEXT_ECHO))
     {
-	sru_warning (_("spconf:Failed to set text echo value:%s\n"),text_echo);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set text echo value: %s."),text_echo);
     }
 }
 
@@ -369,8 +373,7 @@ spconf_modifiers_set (const gchar *modifiers)
 {
     if (!gnopiconf_set_string (modifiers, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_MODIFIERS))
     {
-	sru_warning (_("spconf:Failed to set modifiers value:%s\n"),modifiers);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set modifiers value: %s."),modifiers);
     }
 }
 
@@ -379,8 +382,7 @@ spconf_cursors_set (const gchar *cursors)
 {
     if (!gnopiconf_set_string (cursors, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_CURSORS))
     {
-	sru_warning (_("spconf:Failed to set cursors value:%s\n"),cursors);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set cursors value: %s."),cursors);
     }
 }
 
@@ -389,8 +391,7 @@ spconf_spaces_set (const gchar *spaces)
 {
     if (!gnopiconf_set_string (spaces, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_SPACES))
     {
-	sru_warning (_("spconf:Failed to set spaces value:%s\n"),spaces);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set spaces value: %s."),spaces);
     }
 }
 
@@ -399,8 +400,7 @@ spconf_dictionary_set (const gchar *dictionary)
 {
     if (!gnopiconf_set_string (dictionary, CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_DICTIONARY_ACTIVE))
     {
-	sru_warning (_("spconf:Failed to set dictionary value:%s\n"),dictionary);
-	sru_warning (_("Recommended to delete ~/.gconf/apps/gnopernicus directories"));
+	sru_warning (_("Failed to set dictionary value: %s."),dictionary);
     }
 }
 
@@ -409,7 +409,7 @@ spconf_play_voice (const gchar *gnopernicus_speaker)
 {
     gnopiconf_unset_key (CONFIG_PATH SPEECH_VOICE_KEY_PATH SPEECH_VOICE_TEST);
     if (!gnopiconf_set_string (gnopernicus_speaker, CONFIG_PATH SPEECH_VOICE_KEY_PATH SPEECH_VOICE_TEST))
-	sru_warning (_("spconf:Failed to set tesable voice%s\n"),gnopernicus_speaker);
+	sru_warning (_("Failed to set testable voice %s."),gnopernicus_speaker);
 }
 
 /**
@@ -466,41 +466,201 @@ spconf_dictionary_get (void)
     return gnopiconf_get_string_with_default (CONFIG_PATH SPEECH_SETTING_KEY_PATH SPEECH_DICTIONARY_ACTIVE, 
 					      DEFAULT_SPEECH_DICTIONARY_ACTIVE);						     
 }
+
 /******************************************************************************/
-SpeechDriver*
+/**
+ * spconf_speech_voice_new:
+ *
+ * Create a new SpeechVoice structure
+ *
+ * return: new allocated structure.
+**/
+static SpeechVoice*
+spconf_speech_voice_new (void)
+{
+    SpeechVoice *sp_voice;
+    
+    sp_voice = (SpeechVoice *) g_new0 (SpeechVoice, 1) ; 
+    
+    sru_assert (sp_voice);
+    
+    return sp_voice;
+}
+
+/**
+ * spconf_speech_voice_free:
+ *
+ * @sp_voice: structure.
+ *
+ * Free Voice strucure
+ *
+**/
+static void
+spconf_speech_voice_free (SpeechVoice *sp_voice)
+{
+    if (!sp_voice)
+	return;
+	
+    g_free (sp_voice->voice_name);
+    g_free (sp_voice->voice_name_descr);
+    g_free (sp_voice);
+    sp_voice = NULL;    
+}
+
+/**
+ * spconf_speech_driver_new:
+ *
+ * Create a new driver structure.
+ *
+ * return: new allocated driver.
+**/
+static SpeechDriver*
 spconf_speech_driver_new (void)
 {
     SpeechDriver *sp_driver;
+    
     sp_driver = (SpeechDriver *) g_new0 (SpeechDriver, 1) ; 
+    
     sru_assert (sp_driver);
+    
     return sp_driver;
 }
 
-void
+/**
+ * spconf_speech_driver_free:
+ *
+ * @sp_driver:
+ *
+ * Free driver structure.
+**/
+static void
 spconf_speech_driver_free (SpeechDriver *sp_driver)
 {
     GSList *tmp;
     
     for (tmp = sp_driver->driver_voice_list; tmp ; tmp = tmp->next)
-	g_free (tmp->data);
+    {
+	spconf_speech_voice_free ((SpeechVoice *)tmp->data);
+	tmp->data = NULL;
+    }
+	
     g_slist_free (sp_driver->driver_voice_list);
+    
     g_free (sp_driver->driver_name);
+    g_free (sp_driver->driver_name_descr);
     g_free (sp_driver);
 }
 
+/**
+ * spconf_driver_list_driver_descr_name_get:
+ *
+ * @driver_name:
+ *
+ * Get a description name of driver. (e.g. "IBM Viavoce GNOME Speech Driver")
+ *
+ * return: driver name. If no longer use need to free it.
+**/
+gchar*
+spconf_driver_list_driver_descr_name_get (const gchar *driver_name)
+{
+    if (!driver_name)
+	return NULL;
+	
+    return g_strdelimit (g_strdup (driver_name), "_", ' ');
+}
+
+/**
+ * spconf_driver_list_voice_descr_name_get:
+ *
+ * @voice_name:
+ *
+ * Get a description name of voice. (e.g. V1 Child1)
+ *
+ * return: voice name. If no longer use need to free it.
+**/
+gchar*
+spconf_driver_list_voice_descr_name_get (const gchar *voice_name)
+{
+    gchar **tokens;
+    gchar *rv = NULL;
+    
+    if (!voice_name)
+	return NULL;
+	
+    /* voice pattern is "V%d voice_name - engine_driver" */
+    
+    tokens = g_strsplit (voice_name, "-", 0);
+    
+    if (tokens[0])
+    {
+	gchar *vname = NULL;
+	
+	/* 4 = strlen ("V%d voice_name ") if voice_name == NULL */
+	vname = g_strchomp (tokens[0]);
+	
+	if (vname[2] == '\0' ||
+	    vname[3] == '\0')
+	{
+	    rv = g_strdup_printf (_("%s no name"), vname);
+	}
+	else
+	{
+	    rv = g_strdup (tokens[0]);
+	}	
+    }
+	
+    g_strfreev (tokens);
+    
+    return rv;
+}
+
+/**
+ * spconf_driver_list_load_speakers:
+ *
+ * @driver_name:
+ *
+ * Return a speaker list for a driver_name driver.
+ *
+ * return: speaker list
+**/
 static GSList*
 spconf_driver_list_load_speakers (const gchar *driver_name)
 {
     GSList *tmp_list = NULL;
+    GSList *voice_list = NULL;
+    GSList *iter = NULL;
     GConfValueType type;
     gchar *path;
     
-    path = g_strdup_printf ("%s%s%s", CONFIG_PATH, SPEECH_DRIVERS_PATH, driver_name);
+    path = g_strdup_printf ("%s%s%s", 
+			    CONFIG_PATH, 
+			    SPEECH_DRIVERS_PATH, 
+			    driver_name);
+			    
     tmp_list = gnopiconf_get_list_with_default (path, NULL, &type);
     g_free (path);
-    return tmp_list;
+    
+    for (iter = tmp_list; iter; iter = iter->next)
+    {
+	SpeechVoice *sp_voice 	    = spconf_speech_voice_new ();
+	sp_voice->voice_name_descr  = spconf_driver_list_voice_descr_name_get (iter->data);
+	sp_voice->voice_name  	    = iter->data;
+	iter->data 		    = NULL;
+	voice_list                  = g_slist_append (voice_list, sp_voice);
+    }
+        
+     g_slist_free (tmp_list);
+    
+    return voice_list;
 }
 
+/**
+ * spconf_driver_list_init:
+ *
+ * Initialize active driver list. (driver and voices)
+ *
+ * return: driver list
+**/
 SpeechDriverListType*
 spconf_driver_list_init (void)
 {
@@ -520,11 +680,13 @@ spconf_driver_list_init (void)
     {
 	if (!list->data)
 	    continue;
+	    
 	driver = spconf_speech_driver_new ();
-	driver->driver_name = list->data;
+	driver->driver_name 	  = list->data;
+	driver->driver_name_descr = spconf_driver_list_driver_descr_name_get (list->data);
+	driver->driver_voice_list = spconf_driver_list_load_speakers (driver->driver_name);
 	list->data = NULL;
-	driver->driver_voice_list = 
-	    spconf_driver_list_load_speakers (driver->driver_name);
+	
 	driver_list = g_slist_append (driver_list, driver);
     }
     
@@ -533,6 +695,13 @@ spconf_driver_list_init (void)
     return driver_list;
 }
 
+/**
+ * spconf_driver_list_free:
+ *
+ * @driver_list:
+ *
+ * Free driver list.
+**/
 SpeechDriverListType*
 spconf_driver_list_free (SpeechDriverListType *driver_list)
 {
@@ -541,9 +710,11 @@ spconf_driver_list_free (SpeechDriverListType *driver_list)
     for (tmp = driver_list; tmp ; tmp = tmp->next)
     {
 	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	
 	spconf_speech_driver_free (driver);
 	tmp->data = NULL;
     }
+    
     g_slist_free (driver_list);    
     
     driver_list = NULL;
@@ -551,12 +722,54 @@ spconf_driver_list_free (SpeechDriverListType *driver_list)
     return driver_list;
 }
 
-/* Return NULL if the driver_name not found or error */
-const SpeechDriver*
-spconf_driver_list_get_driver (SpeechDriverListType *driver_list,
-			       const gchar *driver_name)
+/**
+ * spconf_driver_list_get_driver_name:
+ * 
+ * @driver_list:
+ * @driver_name_descr:
+ *
+ * Return a driver name from description name
+ *
+ * return: const driver name.
+**/
+const gchar*
+spconf_driver_list_get_driver_name (SpeechDriverListType *driver_list,
+				    const gchar *driver_name_descr)
 {
     SpeechDriverListType *tmp;
+    
+    if (!driver_list)
+	return NULL;
+    if (!driver_name_descr)
+	return NULL;
+
+    for (tmp = driver_list ; tmp ; tmp = tmp->next)
+    {
+	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	
+	if (!strcmp (driver->driver_name_descr, driver_name_descr))
+	    return driver->driver_name;
+    }
+    
+    return NULL;
+}
+
+/**
+ * spconf_driver_list_get_driver_name_descr:
+ *
+ * @driver_list:
+ * @driver_name:
+ *
+ * Return a driver description name for drive_name
+ *
+ * return: const driver_name_descr
+**/
+const gchar*
+spconf_driver_list_get_driver_name_descr (SpeechDriverListType *driver_list,
+					 const gchar *driver_name)
+{
+    SpeechDriverListType *tmp;
+    
     if (!driver_list)
 	return NULL;
     if (!driver_name)
@@ -565,18 +778,95 @@ spconf_driver_list_get_driver (SpeechDriverListType *driver_list,
     for (tmp = driver_list ; tmp ; tmp = tmp->next)
     {
 	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	
 	if (!strcmp (driver->driver_name, driver_name))
-	    return driver;
+	    return driver->driver_name_descr;
     }
+    
     return NULL;
 }
 
-/* Return NULL if the voice_name not found or error */
+/**
+ * spconf_driver_list_get_driver_from_name_descr:
+ *
+ * @driver_list:
+ * @driver_name_desc:
+ *
+ * Return a driver whit driver_name_descr.
+ *
+ * return:
+**/
+const SpeechDriver*
+spconf_driver_list_get_driver_from_name_descr (SpeechDriverListType *driver_list,
+					      const gchar *driver_name_descr)
+{
+    SpeechDriverListType *tmp;
+    
+    if (!driver_list)
+	return NULL;
+    if (!driver_name_descr)
+	return NULL;
+
+    for (tmp = driver_list ; tmp ; tmp = tmp->next)
+    {
+	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	
+	if (!strcmp (driver->driver_name_descr, driver_name_descr))
+	    return driver;
+    }
+    
+    return NULL;
+}
+
+/**
+ * spconf_driver_list_get_driver:
+ *
+ * @driver_list:
+ * @driver_name:
+ *
+ * Return driver whit driver_name
+ *
+ * return: NULL if the driver_name not found or error 
+**/
+const SpeechDriver*
+spconf_driver_list_get_driver (SpeechDriverListType *driver_list,
+			       const gchar *driver_name)
+{
+    SpeechDriverListType *tmp;
+    
+    if (!driver_list)
+	return NULL;
+    if (!driver_name)
+	return NULL;
+
+    for (tmp = driver_list ; tmp ; tmp = tmp->next)
+    {
+	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	
+	if (!strcmp (driver->driver_name, driver_name))
+	    return driver;
+    }
+    
+    return NULL;
+}
+
+/**
+ * spconf_driver_list_get_driver_by_voice:
+ * (This function is deprecated)
+ *
+ * @driver_list:
+ * @voice_name:
+ *
+ * Return driver which has a voice.
+ *
+ * return: NULL if the voice_name not found or error. 
+**/
 const SpeechDriver*
 spconf_driver_list_get_driver_by_voice (SpeechDriverListType *driver_list,
 			    		const gchar *voice_name)
 {
     SpeechDriverListType *tmp;
+    
     if (!driver_list)
 	return NULL;
     if (!voice_name)
@@ -585,15 +875,99 @@ spconf_driver_list_get_driver_by_voice (SpeechDriverListType *driver_list,
     for (tmp = driver_list ; tmp ; tmp = tmp->next)
     {
 	SpeechDriver *driver = (SpeechDriver *)tmp->data;
-	GSList *drv_list;
-	for (drv_list = driver->driver_voice_list; drv_list ; drv_list = drv_list->next)
+	GSList *voice_list   = NULL;
+	
+	for (voice_list = driver->driver_voice_list; voice_list ; voice_list = voice_list->next)
 	{
-	    if (!strcmp (drv_list->data, voice_name))
+	    SpeechVoice *speech_voice = (SpeechVoice*)voice_list->data;
+	    
+	    if (!strcmp (speech_voice->voice_name, voice_name))
 		return driver;
 	}
     }
+    
     return NULL;
 }
+
+/**
+ * spconf_driver_list_get_voice_name:
+ *
+ * @driver_list:
+ * @driver_name:
+ * @voice_name_descr:
+ *
+ * Return a voice name, from driver_name driver and which has voice_name_descr.
+ *
+ * return: const
+**/
+const gchar*
+spconf_driver_list_get_voice_name (SpeechDriverListType *driver_list,
+				   const gchar *driver_name,
+			    	   const gchar *voice_name_descr)
+{
+    SpeechDriverListType *tmp;
+    
+    if (!driver_list)
+	return NULL;
+    if (!driver_name)
+	return NULL;
+    if (!voice_name_descr)
+	return NULL;
+
+    for (tmp = driver_list ; tmp ; tmp = tmp->next)
+    {
+	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	GSList *voice_list   = NULL;
+	
+	if (strcmp (driver->driver_name, driver_name))
+	    continue;
+
+	for (voice_list = driver->driver_voice_list; voice_list ; voice_list = voice_list->next)
+	{
+	    SpeechVoice *speech_voice = (SpeechVoice*)voice_list->data;
+	    
+	    if (!strcmp (speech_voice->voice_name_descr, voice_name_descr))
+		return speech_voice->voice_name;
+	}
+    }
+    
+    return NULL;
+}
+
+const SpeechVoice*
+spconf_driver_list_get_speech_voice (SpeechDriverListType *driver_list,
+				     const gchar *driver_name,
+			    	     const gchar *voice_name_descr)
+{
+    SpeechDriverListType *tmp;
+    
+    if (!driver_list)
+	return NULL;
+    if (!driver_name)
+	return NULL;
+    if (!voice_name_descr)
+	return NULL;
+
+    for (tmp = driver_list ; tmp ; tmp = tmp->next)
+    {
+	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	GSList *voice_list   = NULL;
+	
+	if (strcmp (driver->driver_name, driver_name))
+	    continue;
+
+	for (voice_list = driver->driver_voice_list; voice_list ; voice_list = voice_list->next)
+	{
+	    SpeechVoice *speech_voice = (SpeechVoice*)voice_list->data;
+	    
+	    if (!strcmp (speech_voice->voice_name_descr, voice_name_descr))
+		return speech_voice;
+	}
+    }
+    
+    return NULL;
+}
+
 /*****************************************************************************/
 GnopernicusSpeakerListType*
 spconf_gnopernicus_speakers_clone (GnopernicusSpeakerListType* gs_list)
@@ -649,12 +1023,15 @@ spconf_gnopernicus_speakers_remove (GnopernicusSpeakerListType *gs_list,
     sru_return_val_if_fail (gn_speaker, gs_list);
 
     elem = gs_list;
+    
     while (elem)
     {
 	if (elem->data && !strcmp (gn_speaker, (gchar*)elem->data))
 	{
 	    gs_list = g_slist_remove_link (gs_list, elem);
+	    
 	    spconf_gnopernicus_speakers_save (gs_list);
+	    
 	    g_free (elem->data);
 	    g_slist_free (elem);
 	    elem = NULL;
@@ -662,6 +1039,7 @@ spconf_gnopernicus_speakers_remove (GnopernicusSpeakerListType *gs_list,
 	}
 	elem = elem->next;
     }
+    
     return gs_list;
 }
 
@@ -755,22 +1133,28 @@ spconf_speaker_settings_load (const gchar *key)
 	    if (speaker)
 	    {
 		speaker->gnopernicus_speaker = g_strdup (key);	    
-		speaker->driver_name = iter->data;
+		speaker->driver_name 	     = iter->data;
+		speaker->driver_name_descr   = spconf_driver_list_driver_descr_name_get (iter->data);
 		iter->data = NULL;
 		iter = iter->next;
+		
 		if (iter && iter->data)
 		{
-		    speaker->driver_voice = iter->data;
+		    speaker->voice_name       = iter->data;
+		    speaker->voice_name_descr = spconf_driver_list_voice_descr_name_get (iter->data);
 		    iter->data = NULL;
 		    iter = iter->next;
+		    
 		    if (iter && iter->data)
 		    {
 			speaker->volume = atoi((gchar*)iter->data);
 			iter = iter->next;
+			
 			if (iter && iter->data)
 			{
 			    speaker->rate   = atoi((gchar*)iter->data);
 			    iter = iter->next;
+			    
 			    if (iter && iter->data)
 			        speaker->pitch  = atoi((gchar*)iter->data);
 			}
@@ -789,20 +1173,19 @@ spconf_speaker_settings_save (SpeakerSettings *speaker_settings)
 {
     gchar *path;
     GSList *list = NULL;
-    
+
     path = g_strdup_printf ("%s%s", CONFIG_PATH SPEECH_VOICE_PARAM_KEY_PATH,
 			    speaker_settings->gnopernicus_speaker);
     				    
     sru_assert (path);    
-    
-    list = 
-	spconf_create_speaker_item (speaker_settings);
+
+    list = spconf_create_speaker_item (speaker_settings);
+
     gnopiconf_set_list (list, GCONF_VALUE_STRING, path);
     
     g_free (path);	
-    
-    list = 
-	spconf_free_speaker_item (list);
+
+    list = spconf_free_speaker_item (list);
 }
 
 SpeakerSettings*
@@ -813,33 +1196,37 @@ spconf_speaker_settings_new (void)
     new_speaker_table = (SpeakerSettings *) g_new0 (SpeakerSettings, 1) ; 
         
     if (!new_speaker_table) 
-	sru_error (_("Unable to allocate memory"));
+	sru_error (_("Unable to allocate memory."));
     
     return new_speaker_table ; 
 }
 
 
-SpeakerSettings*
-spconf_speaker_settings_free (SpeakerSettings *speaker_settings)
-{
-    if (!speaker_settings) 
-	return NULL;
-    g_free (speaker_settings->gnopernicus_speaker);
-    g_free (speaker_settings->driver_name);
-    g_free (speaker_settings->driver_voice);
-    g_free (speaker_settings);
-    speaker_settings = NULL;
-    return speaker_settings;
-}
-
 SpeakerSettings*
 spconf_speaker_settings_clear (SpeakerSettings *speaker_settings)
 {
     if (!speaker_settings) 
 	return NULL;
+	
     g_free (speaker_settings->gnopernicus_speaker);
     g_free (speaker_settings->driver_name);
-    g_free (speaker_settings->driver_voice);
+    g_free (speaker_settings->driver_name_descr);
+    g_free (speaker_settings->voice_name);
+    g_free (speaker_settings->voice_name_descr);
+    
+    return speaker_settings;
+}
+
+SpeakerSettings*
+spconf_speaker_settings_free (SpeakerSettings *speaker_settings)
+{
+    if (!speaker_settings) 
+	return NULL;
+	    
+    speaker_settings = spconf_speaker_settings_clear (speaker_settings);
+    g_free (speaker_settings);
+    speaker_settings = NULL;
+    
     return speaker_settings;
 }
 
@@ -855,10 +1242,13 @@ spconf_speaker_settings_copy (SpeakerSettings *dest,
     
     dest->gnopernicus_speaker 	= g_strdup (source->gnopernicus_speaker);
     dest->driver_name    	= g_strdup (source->driver_name);
-    dest->driver_voice 		= g_strdup (source->driver_voice);
+    dest->driver_name_descr    	= g_strdup (source->driver_name_descr);
+    dest->voice_name 		= g_strdup (source->voice_name);
+    dest->voice_name_descr	= g_strdup (source->voice_name_descr);
     dest->volume     		= source->volume;
     dest->rate       		= source->rate;
     dest->pitch      		= source->pitch;
+    
     return dest;
 }
 
@@ -882,7 +1272,9 @@ spconf_speaker_settings_list_clone (SpeakerSettingsListType *sp_list)
 	{
 	    new_item->gnopernicus_speaker = g_strdup (item->gnopernicus_speaker);
 	    new_item->driver_name 	  = g_strdup (item->driver_name);
-	    new_item->driver_voice 	  = g_strdup (item->driver_voice);
+	    new_item->driver_name_descr   = g_strdup (item->driver_name_descr);
+	    new_item->voice_name 	  = g_strdup (item->voice_name);
+	    new_item->voice_name_descr    = g_strdup (item->voice_name_descr);
 	    new_item->volume     	  = item->volume;
 	    new_item->rate       	  = item->rate;
 	    new_item->pitch      	  = item->pitch;
@@ -919,7 +1311,7 @@ SpeakerSettingsListType*
 spconf_speaker_settings_list_add (SpeakerSettingsListType *sp_list, 
 			          const gchar 	*gn_speaker,
 			          const gchar 	*driver_name,
-			          const gchar 	*driver_voice,
+			          const gchar 	*voice_name,
 			    	  gint  	volume,
 			          gint  	rate,
 			    	  gint  	pitch)
@@ -927,7 +1319,7 @@ spconf_speaker_settings_list_add (SpeakerSettingsListType *sp_list,
     SpeakerSettings *speaker = NULL;
     SpeakerSettingsListType *elem = NULL;
     
-    sru_return_val_if_fail (driver_voice, sp_list);
+    sru_return_val_if_fail (voice_name, sp_list);
     sru_return_val_if_fail (gn_speaker, sp_list);
 
 	
@@ -942,7 +1334,9 @@ spconf_speaker_settings_list_add (SpeakerSettingsListType *sp_list,
 
     speaker->gnopernicus_speaker = g_strdup (gn_speaker);
     speaker->driver_name 	 = g_strdup (driver_name);
-    speaker->driver_voice 	 = g_strdup (driver_voice);
+    speaker->driver_name_descr 	 = spconf_driver_list_driver_descr_name_get (driver_name);
+    speaker->voice_name 	 = g_strdup (voice_name);
+    speaker->voice_name_descr	 = spconf_driver_list_voice_descr_name_get (voice_name); 
     speaker->volume     	 = volume;
     speaker->rate       	 = rate;
     speaker->pitch      	 = pitch;
@@ -993,20 +1387,24 @@ spconf_speaker_settings_list_remove (SpeakerSettingsListType *sp_list,
 	    if (!strcmp (gn_speaker, item->gnopernicus_speaker))
 	    {
 		gchar *path = NULL;
-		sp_list = 
-		    g_slist_remove_link (sp_list, elem);
+		
+		sp_list = g_slist_remove_link (sp_list, elem);
 		    
 		if (strcmp (NONE_ELEMENT, item->gnopernicus_speaker))
 		{
 		    path = g_strdup_printf ("%s%s", CONFIG_PATH SPEECH_VOICE_PARAM_KEY_PATH,
 					    item->gnopernicus_speaker);
+		    
 		    gnopiconf_unset_key (path);
+		    
 		    spconf_speech_voice_removed (item->gnopernicus_speaker);
+		    
 		    g_free (path);
 		}
 		
 		elem->data = 
 		    spconf_speaker_settings_free ((SpeakerSettings*)elem->data);
+		    
 		g_slist_free (elem);
 		elem = NULL;
 		break;
@@ -1022,6 +1420,7 @@ SpeakerSettingsListType*
 spconf_speaker_settings_list_free (SpeakerSettingsListType *sp_list)
 {
     SpeakerSettingsListType *elem = sp_list;
+    
     if (!sp_list) 
 	return NULL;
     
@@ -1029,6 +1428,7 @@ spconf_speaker_settings_list_free (SpeakerSettingsListType *sp_list)
     {
         elem->data = 
 		spconf_speaker_settings_free ((SpeakerSettings*)elem->data);
+		
 	elem = elem->next;
     }
 	
@@ -1052,10 +1452,12 @@ spconf_speaker_settings_list_load_default (void)
 	{
 	    speaker->gnopernicus_speaker = g_strdup (speakers[iter].gnopernicus_speaker);
 	    speaker->driver_name 	 = g_strdup (speakers[iter].driver_name);
-	    speaker->driver_voice 	 = g_strdup (speakers[iter].driver_voice);
-	    speaker->volume 	= speakers[iter].volume;
-	    speaker->rate       = speakers[iter].rate;
-	    speaker->pitch      = speakers[iter].pitch;
+	    speaker->driver_name_descr 	 = g_strdup (speakers[iter].driver_name_descr);
+	    speaker->voice_name 	 = g_strdup (speakers[iter].voice_name);
+	    speaker->voice_name_descr	 = g_strdup (speakers[iter].voice_name_descr);
+	    speaker->volume 		 = speakers[iter].volume;
+	    speaker->rate                = speakers[iter].rate;
+	    speaker->pitch               = speakers[iter].pitch;
 	    
 	    spconf_speaker_settings_save (speaker);
 	    
@@ -1077,6 +1479,7 @@ spconf_speaker_settings_list_load_from_gconf (GnopernicusSpeakerListType *gnoper
     {
 	SpeakerSettings *speaker =
 	    spconf_speaker_settings_load ((gchar*)elem->data);
+	    
 	if (speaker)
 	{
 	    speakers_settings = 
@@ -1099,16 +1502,19 @@ spconf_speaker_settings_list_save (SpeakerSettingsListType *sp_list)
     for (elem = sp_list; elem ; elem = elem->next)
     {
 	SpeakerSettings *item = (SpeakerSettings*)elem->data;
+	
 	if (item)
 	    spconf_speaker_settings_save (item);
     }
 }
 
+
 /******************************************************************************/
 GSList*
 spconf_free_slist_with_char_data (GSList *list)
 {
     GSList *elem = NULL;
+    
     if (!list) 
 	return NULL;
     
@@ -1117,6 +1523,7 @@ spconf_free_slist_with_char_data (GSList *list)
 	
     g_slist_free (list);
     list = NULL;
+    
     return list;
 }
 
@@ -1134,8 +1541,9 @@ spconf_driver_list_for_combo_get (SpeechDriverListType *speech_driver_list,
     for (tmp = speech_driver_list; tmp ; tmp = tmp->next)
     {
     	SpeechDriver *driver = (SpeechDriver *)tmp->data;
+	
 	driver_list = 
-	    g_list_append (driver_list, g_strdup (driver->driver_name));
+	    g_list_append (driver_list, g_strdup (driver->driver_name_descr));
     }
     
     return driver_list;
@@ -1152,17 +1560,20 @@ spconf_voice_list_for_combo_get (SpeechDriverListType	*speech_driver_list,
     
     if (item)
 	voice_list = g_list_append (voice_list, g_strdup (item));
-	
+
     sdriver =
-	spconf_driver_list_get_driver (speech_driver_list, driver);
-    
+	spconf_driver_list_get_driver_from_name_descr (speech_driver_list,
+							driver);
+
     if (!sdriver)
 	return voice_list;
 
     for (tmp = sdriver->driver_voice_list; tmp ; tmp = tmp->next)
     {
+	SpeechVoice *speech_voice = (SpeechVoice*)tmp->data; 
+	
 	voice_list = 
-	    g_list_append (voice_list, g_strdup (tmp->data));
+	    g_list_append (voice_list, g_strdup (speech_voice->voice_name_descr));
     }
     
     return voice_list;
@@ -1183,12 +1594,15 @@ GSList*
 spconf_create_speaker_item (SpeakerSettings *item)
 {
     GSList *list = NULL;
+    
     sru_return_val_if_fail (item, NULL);
-    list = g_slist_append (list, g_strdup (item->driver_name));
-    list = g_slist_append (list, g_strdup (item->driver_voice));
+    
+    list = g_slist_append (list, g_strdup (item->driver_name));  
+    list = g_slist_append (list, g_strdup (item->voice_name));   
     list = g_slist_append (list, g_strdup_printf ("%d", item->volume));
     list = g_slist_append (list, g_strdup_printf ("%d", item->rate));
     list = g_slist_append (list, g_strdup_printf ("%d", item->pitch));
+    
     return list;
 }
 
@@ -1228,10 +1642,12 @@ spconf_set_default_voices (gboolean force)
 	    {
 		speaker->gnopernicus_speaker 	= g_strdup (speakers[iter].gnopernicus_speaker);
 		speaker->driver_name 		= g_strdup (speakers[iter].driver_name);
-		speaker->driver_voice 		= g_strdup (speakers[iter].driver_voice);
-		speaker->volume     = speakers[iter].volume;
-		speaker->rate       = speakers[iter].rate;
-		speaker->pitch      = speakers[iter].pitch;
+		speaker->driver_name_descr	= g_strdup (speakers[iter].driver_name_descr);
+		speaker->voice_name 		= g_strdup (speakers[iter].voice_name);
+		speaker->voice_name_descr	= g_strdup (speakers[iter].voice_name_descr);
+		speaker->volume     		= speakers[iter].volume;
+		speaker->rate       		= speakers[iter].rate;
+		speaker->pitch      		= speakers[iter].pitch;
 
 		spconf_speaker_settings_save (speaker);	    
 		
@@ -1252,8 +1668,11 @@ DictionaryListType*
 spconf_dictionary_load_default (void)
 {
     DictionaryListType *list = NULL;
+    
     list = g_slist_append (list, g_strdup (DEFAULT_SPEECH_DICTIONARY_ENTRY));
+    
     spconf_dictionary_save (list);
+    
     return list;
 }
 
@@ -1291,15 +1710,19 @@ DictionaryListType*
 spconf_dictionary_free (DictionaryListType *list)
 {
     DictionaryListType *elem = NULL;
+    
     if (!list)
 	return NULL;
+	
     for (elem = list; elem ; elem = elem->next)
     {
 	g_free ((gchar*)elem->data);
 	elem->data = NULL;
     }
+    
     g_slist_free (list);
     list = NULL;
+    
     return list;
 }
 
@@ -1316,13 +1739,16 @@ spconf_dictionary_split_entry (const gchar *entry,
     sru_return_val_if_fail (entry, FALSE);
     
     separator = g_utf8_strchr (entry, -1, '<');
+    
     if (separator && *(g_utf8_find_next_char (separator, NULL)) == '>')
     {
-	*word = g_strndup (entry, separator - entry);
+	*word     = g_strndup (entry, separator - entry);
 	separator = g_utf8_find_next_char (separator, NULL);
 	*replace  = g_strdup (g_utf8_find_next_char (separator,NULL));
+	
 	return TRUE;
     }
+    
     return FALSE;
 }
 
@@ -1400,6 +1826,7 @@ spconf_dictionary_add_word (DictionaryListType *list,
 
     list = g_slist_append (list,
 	    spconf_dictionary_create_entry (word, replace));
+	    
     return list;
 }
 
@@ -1408,13 +1835,17 @@ spconf_dictionary_remove_word (DictionaryListType *list,
 			      const gchar *word)
 {
     DictionaryListType *elem = NULL;
+    
     sru_return_val_if_fail (word, list);
     sru_return_val_if_fail (list, list);
 
     if ((elem = spconf_dictionary_find_word (list, word)) == NULL)
 	return list;
+	
     list = g_slist_remove_link (list, elem);
+    
     spconf_dictionary_free (elem);
+    
     return list;
 }
 
