@@ -583,6 +583,33 @@ src_braille_show (gchar *message)
     g_free(text);
 }
 
+gboolean
+src_braille_translation_table_exist (const gchar *table)
+{
+    gchar *fn = NULL;
+    gboolean rv = FALSE;			
+    
+    /* derive the TT file name from the language name */
+    fn = g_strdup_printf ("%s.a2b", table);
+    /* test the file */
+    if (g_file_test (fn, G_FILE_TEST_EXISTS)) 
+    {
+	rv = TRUE;
+    }
+    else
+    {
+	gchar *dir_fn = NULL;
+	dir_fn = g_strconcat (BRAILLE_TRANS_TABLES_DIR,fn,NULL);
+	if (g_file_test (dir_fn, G_FILE_TEST_EXISTS)) 
+	    rv = TRUE;
+	g_free (dir_fn);
+    }
+    g_free (fn);
+    
+    return rv;
+}
+
+
 gboolean
 src_braille_restart ()
 {
