@@ -643,11 +643,23 @@ src_process_config_changed_for_braille (SRConfigStructure *config)
 	{
 	    if (!strcmp (config->newvalue, src_braille_translation_table))
 		return;
-	    g_free (src_braille_translation_table);
-		
-	    src_braille_translation_table = g_strdup (config->newvalue);				    
 	    
-	    restart = TRUE;
+	    if (src_braille_translation_table_exist (config->newvalue))
+	    {
+		g_free (src_braille_translation_table);
+		
+		src_braille_translation_table = g_strdup (config->newvalue);				    
+		
+		restart = TRUE;
+	    }
+	    else
+	    {
+		gchar *value =  DEFAULT_BRAILLE_TRANSLATION;
+		
+		srconf_set_config_data (BRAILLE_TRANSLATION, CFGT_STRING, value, CFGM_BRAILLE);		
+		
+		restart = FALSE;
+	    }	    
 	}
 	else
 	{
@@ -2375,6 +2387,11 @@ src_get_defaults ()
 	gchar *default_braille_transaltion_table = DEFAULT_BRAILLE_TRANSLATION;
 	GET_BRAILLE_CONFIG_DATA_WITH_DEFAULT (BRAILLE_TRANSLATION, CFGT_STRING, 
 		&src_braille_translation_table, default_braille_transaltion_table);
+		
+	if (!src_braille_translation_table_exist (src_braille_translation_table))
+	{
+	    srconf_set_config_data (BRAILLE_TRANSLATION, CFGT_STRING, DEFAULT_BRAILLE_TRANSLATION, CFGM_BRAILLE);
+	}	    
     }
     
     {
@@ -2409,6 +2426,7 @@ src_get_defaults ()
 	    &src_use_braille, &src_use_braille);
     }
     
+    
     if ((gint)src_use_braille_monitor == -1)
     {
 	GET_SRCORE_CONFIG_DATA_WITH_DEFAULT (SRCORE_BRAILLE_MONITOR_ACTIVE, CFGT_BOOL, 
