@@ -1307,6 +1307,7 @@ static struct
         {"frame",			"FRM",	"FRAME",		"Frame",		},
         {"glass-pane",			"GPN",	"GLASS PANE",		"Glass pane",		},
 	{"HTML-container",		"HTM",	"HTML CONTAINER",	"H T M L container",	},
+	{"hyper link",			"LNK",	"LINK",			"link",		},
         {"icon",			"ICO",	"ICON",			"Icon",			},
         {"internal-frame",		"IFR",	"INTERNAL FRAME",	"Internal frame",	},
 	{"label",			"LBL",	"LABEL",		"Label",		},
@@ -1390,7 +1391,10 @@ src_presentation_get_role_name_for_braille (const gchar *role)
 
     index = src_presentation_get_role_index (role);
 
+    if (index)
     return src_presentation_role_pres[index].braille_short;
+    else
+	return role;
 }
 
 static G_CONST_RETURN gchar* 
@@ -1402,7 +1406,10 @@ src_presentation_get_role_name_for_speech (const gchar *role)
 
     index = src_presentation_get_role_index (role);
 
+    if (index)
     return src_presentation_role_pres[index].speech_short;
+    else
+	return role;
 }
 
 
@@ -1650,8 +1657,8 @@ src_presentation_get_info_from_sro (SRObject *obj,
 
 static gchar*
 src_presentation_present_sro_for_device_ (SRObject *obj,
-					 const gchar *device,
-					 SRCPresentationItem **chunk)
+					  const gchar *device,
+					  SRCPresentationItem **chunk)
 {
     gint i;
     gchar *rv;
@@ -1686,33 +1693,42 @@ src_presentation_present_sro_for_device_ (SRObject *obj,
 		    
 		    add = g_strdup ("");		    
 		    sro_get_children_count (obj, &cnt);
-		    for (j = 0; j < cnt; j++)
+		    if (!sro_manages_descendants (obj)) 
 		    {
-			SRObject *child;
-			gchar *tmp, *tmp2, *tmp3;
-
-			if (!sro_get_i_child (obj, j, &child))
-			    continue;
-			
-			tmp = add;
-			tmp2 = src_presentation_present_sro_for_device_ (child, device, (SRCPresentationItem **)chunk[i]->data1);
-			
-			tmp3 = NULL;
-			if (j != cnt - 1 && chunk[i]->data2)
+		        for (j = 0; j < cnt; j++)
 			{
-			    gchar *tmp4;
-			    tmp4 = g_strdup_printf ("dot%s", (gchar *)chunk[i]->data2);
-			    if (tmp4)
+			    SRObject *child;
+			    gchar *tmp, *tmp2, *tmp3;
+			  
+			    if (!sro_get_i_child (obj, j, &child))
+			      continue;
+			    
+			    tmp = add;
+			    tmp2 = src_presentation_present_sro_for_device_ (child, device, (SRCPresentationItem **)chunk[i]->data1);
+			    
+			    tmp3 = NULL;
+			    if (j != cnt - 1 && chunk[i]->data2)
 			    {
-				tmp3 = src_xml_make_part ("DOTS", NULL, tmp4);
-				g_free (tmp4);
+			        gchar *tmp4;
+				tmp4 = g_strdup_printf ("dot%s", (gchar *)chunk[i]->data2);
+				if (tmp4)
+				{
+				    tmp3 = src_xml_make_part ("DOTS", NULL, tmp4);
+				    g_free (tmp4);
+				}
 			    }
+			    add = g_strconcat (add, tmp2, tmp3, NULL);
+			    g_free (tmp);
+			    g_free (tmp2);
+			    g_free (tmp3);
+			    sro_release_reference (child);
 			}
-			add = g_strconcat (add, tmp2, tmp3, NULL);
-			g_free (tmp);
-			g_free (tmp2);
-			g_free (tmp3);
-			sro_release_reference (child);
+		    }
+		    else
+		    {
+		      /* FIXME: presentation of first N visible children?
+			 Or just a notation of "Object contains %d children" ?
+		      */
 		    }
 		}
 		break;
@@ -1887,10 +1903,8 @@ src_presentation_present_sro_for_magnifier (SRObject *obj,
 		}
 		else if (strncmp (string + 1, "text:char", 9) == 0)
 		{
-		    SRLong index;
-		    
-		    sro_text_get_caret_offset (obj, &index, SR_INDEX_CONTAINER);
-		    sro_text_get_location_at_index (obj, index, &location, SR_INDEX_CONTAINER);
+		    sro_text_get_caret_location (obj, SR_COORD_TYPE_SCREEN, 
+				&location, SR_INDEX_CONTAINER);
 		}
 		else if (strncmp (string + 1, "object", 6) == 0)
 		{
