@@ -70,11 +70,15 @@
 #define SRL_EVENT_NAME_NAME_CHANGED		"object:property-change:accessible-name"
 
 #define SRL_TOOLKIT_MOZILLA			"mozilla"
+#define SRL_APP_METACITY			"metacity"
 
 typedef enum _SRLEventType
 {
     SRL_EVENT_UNKNOWN = 0,
     SRL_EVENT_FOCUS,
+    SRL_EVENT_FOCUS1, /* some events are reported as "focus:",
+			but all other events for those objects 
+			should not be reported. (eg ALT+CTRL+TAB) */
     SRL_EVENT_LINK_SELECTED,
     SRL_EVENT_TEXT_CARET_MOVED,
     SRL_EVENT_TEXT_CHANGED_INSERT,
@@ -668,6 +672,8 @@ srle_get_reason (SRLEvent *event)
 	return SRL_EVENT_NAME_FOCUS;
     if (srle_has_type (event, SRL_EVENT_WINDOW_RENAME))
 	return SRL_EVENT_NAME_WINDOW_RENAME;
+    if (srle_has_type (event, SRL_EVENT_FOCUS1))
+	return SRL_EVENT_NAME_FOCUS;
 
     srl_assert_not_reached ();
     return SRL_EVENT_NAME_UNKNOWN;
@@ -695,6 +701,7 @@ srl_is_focus_event (SRLEvent *event)
     srl_assert (event);
     
     return  event->type == SRL_EVENT_FOCUS		||
+	    event->type == SRL_EVENT_FOCUS1		||
 	    event->type == SRL_EVENT_LINK_SELECTED	||
 	    event->type == SRL_EVENT_TEXT_CARET_MOVED	||
 	    event->type == SRL_EVENT_TEXT_CHANGED_INSERT||
@@ -984,6 +991,35 @@ srle_is_for_focused_acc (SRLEvent *event)
     return srl_last_focus == srle_get_acc (event);
 }
 
+static gboolean
+srl_event_is_for_metacity (SRLEvent *event)
+{
+    gboolean rv = FALSE;
+    Accessible *acc;
+
+    srl_assert (event);
+
+    acc = (Accessible*) srle_get_acc (event);
+    Accessible_ref (acc);
+    while (acc && !Accessible_isApplication (acc))
+    {
+	Accessible *parent = Accessible_getParent (acc);
+	Accessible_unref (acc);
+	acc = parent;
+    }
+    if (acc)
+    {
+    	gchar *name = Accessible_getName (acc);
+	if (name && strcmp (name, SRL_APP_METACITY) == 0)
+	    rv = TRUE;
+	SPI_freeString (name);
+	Accessible_unref (acc);
+    }
+
+    return rv;
+}
+
+
 static gboolean
 srle_change_type (SRLEvent *event)
 {
@@ -1012,12 +1048,18 @@ srle_change_type (SRLEvent *event)
     }
     else if (srle_has_type (event, SRL_EVENT_NAME_CHANGED))
     {
-	Accessible *parent = Accessible_getParent (event->acc_ev->source);
-	if (parent)
+	if (srle_acc_has_role (event, SPI_ROLE_STATUS_BAR) &&
+	    srl_event_is_for_metacity (event))
+	    event->type = SRL_EVENT_FOCUS1;
+	else
 	{
-	    if (Accessible_isApplication (parent))
-		event->type = SRL_EVENT_WINDOW_RENAME;
-	    Accessible_unref (parent);
+	    Accessible *parent = Accessible_getParent (event->acc_ev->source);
+	    if (parent)
+	    {
+		if (Accessible_isApplication (parent))
+		    event->type = SRL_EVENT_WINDOW_RENAME;
+		Accessible_unref (parent);
+	    }
 	}
     }
     else if (srle_has_type (event, SRL_EVENT_LINK_SELECTED))
@@ -1259,6 +1301,8 @@ srl_event_is_reported (SRLEvent *event)
 	rv = TRUE;
     else if (srle_has_type (event, SRL_EVENT_FOCUS))
 	rv = TRUE;
+    else if (srle_has_type (event, SRL_EVENT_FOCUS1))
+	rv = TRUE;
     else if (srle_is_for_focused_acc (event))
     {
 	if (srle_has_type (event, SRL_EVENT_STATE_EXPANDED)) 
@@ -1282,9 +1326,7 @@ srl_event_is_reported (SRLEvent *event)
 	    rv = srl_selection_event_is_reported (event);
 	else if (srle_has_type (event, SRL_EVENT_VALUE_CHANGED))
 	    rv = srle_acc_has_role (event, SPI_ROLE_SPIN_BUTTON);
-
     }
-    
 
     if (srle_has_type (event, SRL_EVENT_WINDOW_DEACTIVATE))
 	rv = FALSE;
@@ -1308,13 +1350,11 @@ srl_event_is_reported (SRLEvent *event)
 }
 
 static gboolean
-srl_set_last_focus (SRLEvent *event)
+srl_set_last_focus (Accessible *acc)
 {
-    srl_assert (event);
-
     if (srl_last_focus)
 	Accessible_unref (srl_last_focus);
-    srl_last_focus = (Accessible*) srle_get_acc (event);
+    srl_last_focus = acc;
     if (srl_last_focus)
 	Accessible_ref (srl_last_focus);
     return TRUE;
@@ -1333,7 +1373,10 @@ srl_process_event (SRLEvent *event)
     process = srl_event_is_reported (event);
 
     if (srle_has_type (event, SRL_EVENT_FOCUS))
-	srl_set_last_focus (event);
+	srl_set_last_focus ((Accessible*)srle_get_acc (event));
+    else if (srle_has_type (event, SRL_EVENT_FOCUS1))
+	srl_set_last_focus (NULL);
+
 /* FIXME: 
     if (srle_has_type (event, SRL_EVENT_WINDOW_DEACTIVATE))
 	report no focus if no other event */
